unit lglcd;

(*

  Translated from lglcd.h (LCDSDK_3.01.180) by delphifreak
  mailto: delphifreak[at]fantasymail[dot]de

  All functions declared in this unit starting with "lgLcd" are documented in
  the official documentation. All others are documented in this unit.

  All structures and types declared in the original lglcd.h are renamed, adapted
  to Delphi standards. So types are declared with a leading "T" (type) or "P"
  (pointer) in front of their original and documented names.

  Most of the comments here are taken over from the original. I added just some
  in my code parts.

  Notice:
    If you want to handle possible exceptions during the initialization of this
    unit yourself, then add "lglcd_NoAutoInit" to your conditional defines and
    initialize/finalize this unit yourself using the functions
    "lglcd_initialize" and "lglcd_finalize".
    If you prefer using just the boolean results instead of the exceptions of
    those functions, then add "lglcd_NoExceptions" to your conditional defines
    too.

*)

//Original Comment from lglcd.h:
(*

  lglcd.h

  library definition for lglcd.a
  part of lglcd for Microsoft(R) Windows(R)

  The Logitech LCD SDK, including all acompanying documentation,
  is protected by intellectual property laws.  All use of the Logitech
  LCD SDK is subject to the License Agreement found in the
  "ReadMe License Agreement" file and in the Reference Manual.  All rights
  not expressly granted by Logitech are reserved.


  01/14/2005    1.00    initial draft
  02/23/2005    1.01    added callbacks, implemented changes as discussed
  02/08/2006    1.02    added call to set foreground, sync update with confirmation
  05/29/2006    1.03    Added device family feature
  12/03/2007    3.00    Added support for color devices and partial event notifications (arrival and removal)
  10/03/2008    3.01    Added more event notifications
  10/27/2008    3.01    Deprecated enumeration and index-based open; Applets should use lgLcdOpenByType() (and
                        possibly notifications)

*)

interface

uses
  Windows, Registry, SysUtils;

//******************************************************************************
// Constants
//******************************************************************************
const
  // Invalid handle definitions
  LGLCD_INVALID_CONNECTION = -1;
  LGLCD_INVALID_DEVICE     = -1;

// Common Soft-Buttons available through the SDK
  LGLCDBUTTON_LEFT   = $00000100;
  LGLCDBUTTON_RIGHT  = $00000200;
  LGLCDBUTTON_OK     = $00000400;
  LGLCDBUTTON_CANCEL = $00000800;
  LGLCDBUTTON_UP     = $00001000;
  LGLCDBUTTON_DOWN   = $00002000;
  LGLCDBUTTON_MENU   = $00004000;

  // Soft-Button masks. Kept for backwards compatibility
  LGLCDBUTTON_BUTTON0 = $00000001;
  LGLCDBUTTON_BUTTON1 = $00000002;
  LGLCDBUTTON_BUTTON2 = $00000004;
  LGLCDBUTTON_BUTTON3 = $00000008;
  LGLCDBUTTON_BUTTON4 = $00000010;
  LGLCDBUTTON_BUTTON5 = $00000020;
  LGLCDBUTTON_BUTTON6 = $00000040;
  LGLCDBUTTON_BUTTON7 = $00000080;


  LGLCD_BMP_FORMAT_160x43x1 = $00000001;
  LGLCD_BMP_FORMAT_QVGAx32  = $00000003;
  LGLCD_BMP_WIDTH           = 160;
  LGLCD_BMP_HEIGHT          = 43;
  LGLCD_BMP_BPP             = 1;
  LGLCD_BW_BMP_WIDTH        = 160;
  LGLCD_BW_BMP_HEIGHT       = 43;
  LGLCD_BW_BMP_BPP          = 1;
  LGLCD_QVGA_BMP_WIDTH      = 320;
  LGLCD_QVGA_BMP_HEIGHT     = 240;
  LGLCD_QVGA_BMP_BPP        = 4;

  // Priorities
  LGLCD_PRIORITY_IDLE_NO_SHOW = 0;
  LGLCD_PRIORITY_BACKGROUND   = 64;
  LGLCD_PRIORITY_NORMAL       = 128;
  LGLCD_PRIORITY_ALERT        = 255;

  // Declared as functions later
(*  LGLCD_SYNC_UPDATE(priority)                 (0x80000000 | (priority))
  LGLCD_SYNC_COMPLETE_WITHIN_FRAME(priority)  (0xC0000000 | (priority))
  LGLCD_ASYNC_UPDATE(priority)                (priority)*)

  // Foreground mode for client applications
  LGLCD_LCD_FOREGROUND_APP_NO  = 0;
  LGLCD_LCD_FOREGROUND_APP_YES = 1;

  // Device family definitions
  LGLCD_DEVICE_FAMILY_BW_160x43_GAMING            = $00000001;
  LGLCD_DEVICE_FAMILY_KEYBOARD_G15                = $00000001;
  LGLCD_DEVICE_FAMILY_BW_160x43_AUDIO             = $00000002;
  LGLCD_DEVICE_FAMILY_SPEAKERS_Z10                = $00000002;
  LGLCD_DEVICE_FAMILY_JACKBOX                     = $00000004;
  LGLCD_DEVICE_FAMILY_BW_160x43_BASIC             = $00000008;
  LGLCD_DEVICE_FAMILY_LCDEMULATOR_G15             = $00000008;
  LGLCD_DEVICE_FAMILY_RAINBOW                     = $00000010;
  LGLCD_DEVICE_FAMILY_QVGA_BASIC                  = $00000020;
  LGLCD_DEVICE_FAMILY_QVGA_GAMING                 = $00000040;
  LGLCD_DEVICE_FAMILY_GAMEBOARD_G13               = $00000080;
  LGLCD_DEVICE_FAMILY_OTHER                       = $80000000;

  // Combinations of device families (device clans?)
  LGLCD_DEVICE_FAMILY_ALL_BW_160x43               = LGLCD_DEVICE_FAMILY_BW_160x43_GAMING or
                                                    LGLCD_DEVICE_FAMILY_BW_160x43_AUDIO or
                                                    LGLCD_DEVICE_FAMILY_JACKBOX or
                                                    LGLCD_DEVICE_FAMILY_BW_160x43_BASIC or
                                                    LGLCD_DEVICE_FAMILY_RAINBOW or
                                                    LGLCD_DEVICE_FAMILY_GAMEBOARD_G13;

  LGLCD_DEVICE_FAMILY_ALL_QVGA                    = LGLCD_DEVICE_FAMILY_QVGA_BASIC or
                                                    LGLCD_DEVICE_FAMILY_QVGA_GAMING;

  LGLCD_DEVICE_FAMILY_ALL                         = LGLCD_DEVICE_FAMILY_ALL_BW_160x43 or
                                                    LGLCD_DEVICE_FAMILY_ALL_QVGA;

  // Capabilities of applets connecting to LCD Manager.
  LGLCD_APPLET_CAP_BASIC                          = $00000000;
  LGLCD_APPLET_CAP_BW                             = $00000001;
  LGLCD_APPLET_CAP_QVGA                           = $00000002;

  // Notifications sent by LCD Manager to applets connected to it.
  LGLCD_NOTIFICATION_DEVICE_ARRIVAL               = $00000001;
  LGLCD_NOTIFICATION_DEVICE_REMOVAL               = $00000002;
  LGLCD_NOTIFICATION_CLOSE_CONNECTION             = $00000003;
  LGLCD_NOTIFICATION_APPLET_DISABLED              = $00000004;
  LGLCD_NOTIFICATION_APPLET_ENABLED               = $00000005;
  LGLCD_NOTIFICATION_TERMINATE_APPLET             = $00000006;

  // Device types used in notifications
  LGLCD_DEVICE_BW                                 = $00000001;
  LGLCD_DEVICE_QVGA                               = $00000002;

//******************************************************************************
// Callback Function Prototypes
//******************************************************************************
type
  // Callback used to notify client of soft button change
  TlgLcdOnSoftButtonsCB = function(device: Integer;
                                   dwButtons: DWORD;
                                   pContext: Pointer
                                  ): DWORD; stdcall;

  // Callback used to allow client to pop up a "configuration panel"
  TlgLcdOnConfigureCB = function(connection: Integer;
                                 pContext: Pointer
                                ): DWORD; stdcall;

  // Callback used to notify client of events, such as device arrival, ...
  // Not implemented as of 1.03 release.
  TlgLcdOnNotificationCB = function(connection: Integer;
                                    pContext: Pointer;
                                    notificationCode: DWORD;
                                    notifyParam1,
                                    notifyParam2,
                                    notifyParam3,
                                    notifyParam4: DWORD
                                   ): DWORD; stdcall;

//******************************************************************************
// Structures
//******************************************************************************
type
  PlgLcdDeviceDesc = ^TlgLcdDeviceDesc;
  TlgLcdDeviceDesc = record
    Width,
    Height,
    Bpp,
    NumSoftButtons: DWORD;
  end;

  PlgLcdDeviceDescExW = ^TlgLcdDeviceDescExW;
  TlgLcdDeviceDescExW = record
    deviceFamilyId: DWORD;
    deviceDisplayName: array [0..MAX_PATH-1] of WideChar;
    Width,             // # of pixels (horizontally) on the LCD
    Heigth,            // # of pixels (lines) on the LCD
    Bpp,               // # of bits per pixel (1,8,16,24,...)
    NumSoftButtons,
    Reserved1,
    Reserved2: DWORD;
  end;

  PlgLcdDeviceDescExA = ^TlgLcdDeviceDescExA;
  TlgLcdDeviceDescExA = record
    deviceFamilyId: DWORD;
    deviceDisplayName: array [0..MAX_PATH-1] of AnsiChar;
    Width,
    Heigth,
    Bpp,
    NumSoftButtons,
    Reserved1,
    Reserved2: DWORD;
  end;
  PlgLcdDeviceDescEx = ^TlgLcdDeviceDescEx;
  TlgLcdDeviceDescEx = TlgLcdDeviceDescExA;

  PlgLcdBitmapHeader = ^TlgLcdBitmapHeader;
  TlgLcdBitmapHeader = record
    Format: DWORD;
  end;

  PlgLcdBitmap160x43x1 = ^TlgLcdBitmap160x43x1;
  TlgLcdBitmap160x43x1 = record
    hdr: TlgLcdBitmapHeader;
    pixels: array [0..(LGLCD_BMP_WIDTH * LGLCD_BMP_HEIGHT * LGLCD_BMP_BPP) - 1] of Byte;
  end;

  PlgLcdBitmap160x43x1_ = ^TlgLcdBitmap160x43x1_;
  TlgLcdBitmap160x43x1_ = record
    hdr: TlgLcdBitmapHeader;
    pixels: array [0..LGLCD_BMP_WIDTH-1] of array [0..LGLCD_BMP_HEIGHT-1] of Byte;
  end;

  PlgLcdBitmapQVGAx32 = ^TlgLcdBitmapQVGAx32;
  TlgLcdBitmapQVGAx32 = record
    hdr: TlgLcdBitmapHeader; // Format = LGLCD_BMP_FORMAT_QVGAx32
    pixels: array [0..(LGLCD_QVGA_BMP_WIDTH * LGLCD_QVGA_BMP_HEIGHT * LGLCD_QVGA_BMP_BPP) - 1] of Byte;
  end;

  // Generic bitmap for use by both color and BW applets
  PlgLcdBitmap = ^TlgLcdBitmap;
  TlgLcdBitmap = record
    case Integer of
      0: (hdr: TlgLcdBitmapHeader);          // provides easy access to the header
      1: (bmp_mono: TlgLcdBitmap160x43x1);   // B/W bitmap data
      2: (bmp_qvga32: TlgLcdBitmapQVGAx32);  // Color bitmap data
  end;

  PlgLcdConfigureContext = ^TlgLcdConfigureContext;
  TlgLcdConfigureContext = record
    // Set to NULL if not configurable
    configCallback: TlgLcdOnConfigureCB;
    configContext: Pointer;
  end;

  PlgLcdNotificationContext = ^TlgLcdNotificationContext;
  TlgLcdNotificationContext = record
    // Set to NULL if not notifiable
    notifyCallback: TlgLcdOnNotificationCB;
    notifyContext: Pointer;
  end;

  PlgLcdConnectContextW = ^TlgLcdConnectContextW;
  TlgLcdConnectContextW = record
    // "Friendly name" display in the listing
    appFriendlyName: LPCWSTR;
    // isPersistent determines whether this connection persists in the list
    isPersistent: BOOL;
    // isAutostartable determines whether the client can be started by
    // LCDMon
    isAutostartable: BOOL;
    onConfigure: TlgLcdConfigureContext;
    // --> Connection handle
    connection: Integer;
  end;

  PlgLcdConnectContextA = ^TlgLcdConnectContextA;
  TlgLcdConnectContextA = record
    // "Friendly name" display in the listing
    appFriendlyName: LPCSTR;
    // isPersistent determines whether this connection persists in the list
    isPersistent: Bool;
    // isAutostartable determines whether the client can be started by
    // LCDMon
    isAutostartable: Bool ;
    onConfigure: TlgLcdConfigureContext;
    // --> Connection handle
    connection: Integer;
  end;
  PlgLcdConnectContext = ^TLgLcdConnectContext;
  TlgLcdConnectContext = TLgLcdConnectContextA;

  PlgLcdConnectContextExW = ^TlgLcdConnectContextExW;
  TlgLcdConnectContextExW = record
    // "Friendly name" display in the listing
    appFriendlyName: LPCWSTR;
    // isPersistent determines whether this connection persists in the list
    isPersistent: BOOL;
    // isAutostartable determines whether the client can be started by
    // LCDMon
    isAutostartable: BOOL;
    onConfigure: TlgLcdConfigureContext;
    // --> Connection handle
    connection: Integer;
    // New additions added in 1.03 revision
    dwAppletCapabilitiesSupported: DWORD; // Or'd combination of LGLCD_APPLET_CAP_... defines
    dwReserved1: DWORD;
    onNotify: TlgLcdNotificationContext;
  end;

  PlgLcdConnectContextExA = ^TlgLcdConnectContextExA;
  TlgLcdConnectContextExA = record
    // "Friendly name" display in the listing
    appFriendlyName: LPCSTR;
    // isPersistent determines whether this connection persists in the list
    isPersistent: BOOL;
    // isAutostartable determines whether the client can be started by
    // LCDMon
    isAutostartable: BOOL;
    onConfigure: TlgLcdConfigureContext;
    // --> Connection handle
    connection: Integer;
    // New additions added in 1.03 revision
    dwAppletCapabilitiesSupported: DWORD; // Or'd combination of LGLCD_APPLET_CAP_... defines
    dwReserved1: DWORD;
    onNotify: TlgLcdNotificationContext;
  end;
  PlgLcdConnectContextEx = ^TlgLcdConnectContextEx;
  TlgLcdConnectContextEx = TlgLcdConnectContextExA;

  PlgLcdSoftbuttonsChangedContext = ^TlgLcdSoftbuttonsChangedContext;
  TlgLcdSoftbuttonsChangedContext = record
    // Set to NULL if no softbutton notifications are needed
    softbuttonsChangedCallback: TlgLcdOnSoftButtonsCB;
    softbuttonsChangedContext: Pointer;
  end;

  PlgLcdOpenContext = ^TlgLcdOpenContext;
  TlgLcdOpenContext = record
    connection,
    // Device index to open
    index: Integer;
    onSoftbuttonsChanged: TlgLcdSoftbuttonsChangedContext;
    // --> Device handle
    device: Integer;
  end;

  PlgLcdOpenByTypeContext = ^TlgLcdOpenByTypeContext;
  TlgLcdOpenByTypeContext = record
    connection,
    // Device index to open
    deviceType: Integer;
    onSoftbuttonsChanged: TlgLcdSoftbuttonsChangedContext;
    // --> Device handle
    device: Integer;
  end;

//******************************************************************************
// Functions
//******************************************************************************

//Functions that were originally declared as constants
function LGLCD_SYNC_UPDATE(priority: DWORD): DWORD;
function LGLCD_SYNC_COMPLETE_WITHIN_FRAME(priority: DWORD): DWORD;
function LGLCD_ASYNC_UPDATE(priority: DWORD): DWORD;

//Initialize and finalize functions declared for external callers, if automated
//initialization and finalization is not preferred
{$IFDEF lglcd_NoAutoInit}
  function lglcd_initialize: boolean;
  function lglcd_finalize: boolean;
{$ENDIF}

//******************************************************************************
// Function Prototypes
//******************************************************************************
var
  // Initialize the library by calling this function.
  lgLcdInit: function: DWORD; stdcall = nil;

  // Must be called to release the library and free all allocated structures.
  lgLcdDeInit: function: DWORD; stdcall = nil;

  // Connect as a client to the LCD subsystem. Provide name to be
  // displayed for user when viewing the user interface of the LCD module,
  // as well as a configuration callback and context, and a flag that states
  // whether this client is startable by LCDMon
  lgLcdConnectW: function(var ctx: TlgLcdConnectContextW): DWORD; stdcall = nil;
  lgLcdConnectA: function(var ctx: TlgLcdConnectContextA): DWORD; stdcall = nil;
  lgLcdConnectExW: function(var ctx: TlgLcdConnectContextExW): DWORD; stdcall = nil;
  lgLcdConnectExA: function(var ctx: TlgLcdConnectContextExA): DWORD; stdcall = nil;

  // Must be called to release the connection and free all allocated resources
  lgLcdDisconnect: function(connection: Integer): DWORD; stdcall = nil;

  // New additions added in 1.03 revision of API. Call this method to setup which device families the applet
  // is interested in. After this call, the applet can use lgLcdEnumerateEx to determine
  // if a device from the device family wanted is found.
  lgLcdSetDeviceFamiliesToUse: function(connection: Integer;
                                        dwDeviceFamiliesSupported: DWORD;
                                        dwReserved1: DWORD
                                       ): DWORD; stdcall = nil;

  // To determine all connected LCD devices supported by this library, and
  // their capabilities. Start with index 0, and increment by one, until
  // the library returns an error (WHICH?).
  lgLcdEnumerate: function(connection: Integer;
                           index: Integer;
                           var description: TlgLcdDeviceDesc
                          ): DWORD; stdcall = nil deprecated;

  lgLcdEnumerateExW: function(connection: Integer;
                              index: Integer;
                              var description: TlgLcdDeviceDescExW
                             ): DWORD; stdcall = nil deprecated;

  lgLcdEnumerateExA: function(connection: Integer;
                              index: Integer;
                              var description: TlgLcdDeviceDescExA
                             ): DWORD; stdcall = nil deprecated;

  // Opens the LCD at position=index. Library sets the device parameter to
  // its internal reference to the device. Calling application provides the
  // device handle in all calls that access the LCD.
  lgLcdOpen: function(var ctx: TlgLcdOpenContext): DWORD; stdcall = nil deprecated;

  // Opens an LCD of the specified type. If no such device is available, returns
  // an error.
  lgLcdOpenByType: function(var ctx: TlgLcdOpenByTypeContext): DWORD; stdcall = nil;

  // Closes the LCD. Must be paired with lgLcdOpen.
  lgLcdClose: function(device: Integer): DWORD; stdcall = nil;

  // Reads the state of the soft buttons for the device.
  lgLcdReadSoftButtons: function(device: Integer;
                                 var buttons: DWORD
                                ): DWORD; stdcall = nil;

  // Provides a bitmap to be displayed on the LCD. The priority field
  // further describes the way in which the bitmap is to be applied.
  lgLcdUpdateBitmap: function(device: Integer;
                              bitmap: PlgLcdBitmapHeader;
                              priority: DWORD
                             ): DWORD; stdcall = nil;

  // Sets the calling application as the shown application on the LCD, and stops
  // any type of rotation among other applications on the LCD.
  lgLcdSetAsLCDForegroundApp: function(device: Integer;
                                       foregroundYesNoFlag: Integer
                                      ): DWORD; stdcall = nil;

implementation

//******************************************************************************
// Functions that were originally declared as constants (what is not possible
// in Delphi; as far as I know :)
//******************************************************************************

function LGLCD_SYNC_UPDATE(priority: DWORD): DWORD;
begin
  Result := $80000000 or priority;
end;

function LGLCD_SYNC_COMPLETE_WITHIN_FRAME(priority: DWORD): DWORD;
begin
  Result := $C0000000 or priority;
end;

function LGLCD_ASYNC_UPDATE(priority: DWORD): DWORD;
begin
  result := priority;
end;

//******************************************************************************
// Internal Stuff
//******************************************************************************
type
  //"Interface"-Table returned by LgLcdGetInterface
  PlgLcdInterfaceTable4 = ^TlgLcdInterfaceTable4;
  TlgLcdInterfaceTable4 = record
    Init,
    DeInit,
    ConnectW,
    ConnectA,
    ConnectExW,
    ConnectExA,
    Disconnect,
    Enumerate,
    EnumerateExW,
    EnumerateExA,
    Open,
    Close,
    ReadSoftButtons,
    UpdateBitmap,
    Unknown14,
    Unknown15,
    Unknown16,
    Unknown17,
    Unknown18,
    Unknown19,
    Unknown20,
    Unknown21,
    Unknown22,
    SetAsLCDForegroundApp,
    SetDeviceFamiliesToUse,
    OpenByType: Pointer;
  end;

var
  //LgLcdGetInterface returns a pointer to an array with function pointers in it
  //what is called "interface", but it can't be imported in Delphi using
  //standard interfaces, due to missing functions (_AddRef, _Release and
  //QueryInterface). If you manage to do it anyway, drop me a mail ;) .
  //This function is used in the SDKs .lib-file (LCDSDK_3.01.180) with dw = 4.
  //Possible values for dw are 1, 2, 3 and 4; the function will return different
  //pointers to a different "interface" for each value of dw. Other values for
  //dw will return a nil/null-pointer.
  lgLcdGetInterface: function(dw: DWORD): Pointer; stdcall = nil;

  //Pointer to the "interface" returned from LgLcdGetInterface
  pInterface: Pointer = nil;
  //Handle to the "LgLcdApi"
  hLib: Cardinal;

//Loads the library, gets the "interface" pointers
function lglcd_initialize: boolean;
var
  reg: TRegistry;
  libpath: string;
begin
  Result := false;

  reg := TRegistry.Create;
  try
    //Get the API-path from the registry
    reg.RootKey := HKEY_LOCAL_MACHINE;
{$IFNDEF lglcd_NoExceptions}
    if not(
{$ENDIF}
      reg.OpenKey('SOFTWARE\Classes\CLSID\{FE750200-B72E-11d9-829B-0050DA1A72D3}\ServerBinary', false)
{$IFNDEF lglcd_NoExceptions}
    )then
      raise Exception.Create('lglcd: Unable to get LgLcdApi library path!')
{$ENDIF};

    libpath := reg.ReadString('');

    //Load the API
    hLib := LoadLibrary(PChar(libpath));
    if not(hLib = 0)then
      //Get the function address for LgLcdGetInterface
      lgLcdGetInterface := GetProcAddress(hLib, 'GetInterface')
{$IFNDEF lglcd_NoExceptions}
    else
      raise Exception.Create('lglcd: Unable to load LgLcdApi!')
{$ENDIF};

    if(Assigned(lgLcdGetInterface))then
      //Get the "interface" pointer
      pInterface := lgLcdGetInterface(4)
{$IFNDEF lglcd_NoExceptions}
    else
      raise Exception.Create('lglcd: Unable to get "GetInterface" function address!')
{$ENDIF};

    if not(pInterface = nil)then
      with PlgLcdInterfaceTable4(pInterface)^ do
      begin
        //Assign the function pointers
{$WARNINGS OFF}
        lgLcdInit                   := Init;
        lgLcdDeInit                 := DeInit;
        lgLcdConnectW               := ConnectW;
        lgLcdConnectA               := ConnectA;
        lgLcdConnectExW             := ConnectExW;
        lgLcdConnectExA             := ConnectExA;
        lgLcdDisconnect             := Disconnect;
        lgLcdSetDeviceFamiliesToUse := SetDeviceFamiliesToUse;
        lgLcdEnumerate              := Enumerate;
        lgLcdEnumerateExW           := EnumerateExW;
        lgLcdEnumerateExA           := EnumerateExA;
        lgLcdOpen                   := Open;
        lgLcdClose                  := Close;
        lgLcdReadSoftButtons        := ReadSoftButtons;
        lgLcdUpdateBitmap           := UpdateBitmap;
        lgLcdSetAsLCDForegroundApp  := SetAsLCDForegroundApp;
        lgLcdOpenByType             := OpenByType;
{$WARNINGS ON}

        Result := true;

        //There are 26 functions allocated in the table, but I can't include all
        //of them here due to lack of time and documentation ^^
        //If you manage to do this, drop me a mail ;)
      end
{$IFNDEF lglcd_NoExceptions}
      else
        raise Exception.Create('lglcd: Unable to get the pointer to the interface table! Please install the latest software for your Logitech hardware!')
{$ENDIF};

  finally
    reg.Free;
  end;
end;

//Frees the library
function lglcd_finalize: boolean;
begin
  Result := false;

  if not(hLib = 0)then
    Result := FreeLibrary(hLib);
end;

{$IFNDEF lglcd_NoAutoInit}
initialization
  try
    lglcd_initialize;
  except
    //Exception handling here, just to show the message, because Delphi doesn't
    //do it during initialization
    on e : Exception do
    begin
      MessageBox(0, PChar(e.Message), 'Exception', MB_ICONERROR);
      raise;
    end;
  end;
finalization
  lglcd_finalize;
{$ENDIF}
end.
